(* ******************************************************************** *)
(*  									*)
(* 	Project		: Ida - Isabelle's documentation assistant	*)
(*  	Author 		: B.Wolff					*)
(*  	Affiliation 	: University of Bremen, 			*)
(*			  Bremen Institute of Safe Systems (BISS)	*)
(*   	This file	: Tactical Operators for structured proof-	*)
(*			  documentation generation			*)
(*  									*)
(*  	$Date: 1997/09/04 15:25:54 $					*)
(*      $Revision: 1.9 $						*)
(*  									*)
(*  ******************************************************************* *)



(* *********************************************************************** *)
(*									   *)
(* General mechanism for protocolling proofs in a docfile		   *)
(*									   *)
(* Tactical Scripts for proof-structuring:				   *)
(*									   *)
(* *********************************************************************** *)


functor Docticals (P : PUNCHER) : 
  sig
    local open P Ref_mgr in
    val CMT        : cmt list -> int -> tactic
    val NR_CMT     : cmt list -> int -> tactic
    val CMTGOAL    : cmt list -> int -> tactic
    val PUNCHGOAL  : bool list -> cmt list -> int -> tactic

    val INDUCT     : string -> string -> int -> tactic
    val HINT       : string -> (int -> tactic) -> int -> tactic
    val GEN_ALL    : string -> int -> tactic

    val CASE_START : tactic
    val CASE       : cmt list -> int -> tactic
    val CASE_END   : tactic

    val CASE_IF    : string -> int -> tactic
    val CASE_ELSE  : int -> tactic
 (* val CASE_END   : tactic  -- as above *)

    val PREM_START : cmt list -> int -> tactic
    val PREM       : cmt list -> int -> tactic
    val PREM_END   : cmt list -> int -> tactic

    val SINCE_START: tactic
    val SINCE      : cmt list -> int -> tactic
    val SINCE_EQ   : cmt list -> int -> tactic
    val SINCE_QED  : cmt list -> int -> tactic
    val SINCE_END  : tactic

    val goal       : theory -> string -> thm list
    val goalw      : theory -> thm list -> string -> thm list
    val prove_goal : theory -> string -> (thm list -> tactic list) -> thm
    val prove_goalw: theory -> thm list -> string 
			-> (thm list -> tactic list) -> thm

    val bind_thm   : string * thm -> unit
    val qed        : string -> unit
    val qed_goal   : string->theory->string-> (thm list -> tactic list) -> unit
    val qed_goalw  : string->theory->thm list->string->
			(thm list->tactic list)->unit
    val QED : string * string * docit_kind * docit_cite_style -> unit
    end
  end =
struct
local open P Induct_chooser Num_mgr Ref_mgr in



val PUSH_NUMS        = () $$ (fn top => (push_nums();all_tac));
val POP_NUMS         = () $$ (fn top => (pop_nums();all_tac));
val INCR_TOP_NUMS    = () $$ (fn top => (incr_top_nums();all_tac));
fun SET_TOP_TAG str  = () $$ (fn top => (set_top_tag str;all_tac));




val CMT 	     = P.CMT;
val NR_CMT 	     = P.NR_CMT;
fun CMTGOAL s n      = P.CMTGOAL(s,[Tab,Tab]) n;
val PUNCHGOAL 	     = P.PUNCHGOAL;


(* ********************************************************************** *)
(* Induction doctical							  *)
(* ********************************************************************** *)



fun INDUCT' ind_name ind_par n top = 
    EVERY[CMTGOAL [Cmt ind_name,Cmt"over", Cmt ind_par, Cmt"."] n,
	  STATE (fn top => (Induct_chooser.INDUCT ind_name ind_par n))];

fun INDUCT ind_name ind_par n = () $$ (INDUCT' ind_name ind_par n);


fun HINT' s tac n top = 
    EVERY[CMT [Cmt"We follow that ", Frm s] n,
	  subgoals_tac [s] n,
          PUSH_NUMS,
	  tac (n+1),
	  POP_NUMS];

fun HINT s tac n = () $$ (HINT' s tac n);



fun GEN_ALL' nam n top= 
    EVERY[CMT [Cmt"We generalize over ",Cmt nam,Cmt ".",Brk] n,
	  res_inst_tac [("x",nam)] spec n];

fun GEN_ALL nam n = () $$ (GEN_ALL' nam n);


(* ********************************************************************** *)
(* Produces documentation layout for case enumerations; also induct's     *)

val CASE_START = PUSH_NUMS;

fun CASE' s n top =
    EVERY[CMTGOAL ((Cmt"Case ":: s) @ [Cmt ":"]) n,
	  INCR_TOP_NUMS];

fun CASE s n = () $$ (CASE' s n);

val CASE_END = POP_NUMS;

(* ********************************************************************** *)
(* Produces documentation layout for a special, two-elementary case       *) 
(* enumerations case_if case_else  case_end - style;                      *)

fun CASE_IF' form n top=
    EVERY[CASE_DIST form n,
          CASE_START,
	  SET_TOP_TAG form,
	  CASE [Frm form] n];

fun CASE_IF s n  = () $$ (CASE_IF' s n);

fun CASE_ELSE' n top =  let val form = top_tag () 
			in  (CASE [Frm ("~("^ form ^")")] n) end;

fun CASE_ELSE n  = () $$ (CASE_ELSE' n);



(* ********************************************************************** *)
(* Produces documentation layout for premisse enumerations;		  *)

fun PREM_START' s n top= 
    EVERY[CMTGOAL (s) n, 
	  CMT [Cmt P.symb_subprf_refine] n,
	  PUSH_NUMS];


fun PREM_START s n = () $$ (PREM_START' s n);


fun PREM' s n top = EVERY[CMTGOAL s n, INCR_TOP_NUMS];

fun PREM s n = () $$ (PREM' s n)


fun PREM_END' s n top = EVERY[CMTGOAL s n,POP_NUMS];

fun PREM_END s n = () $$ (PREM_END' s n);



(* ********************************************************************** *)
(* Produces documentation layout for "since" enumerations;		  *)
(* "dutch style" of proof representation				  *)



val SINCE_START  = PUSH_NUMS;

fun SINCE' s n top = EVERY[P.CMTGOAL (s,[Tab,Cmt P.symb_subprf_refine,Tab]) n,
			   INCR_TOP_NUMS];

fun SINCE s n = () $$ (SINCE' s n);

fun SINCE_EQ' s n top = 
    EVERY[P.CMTGOAL (s,[Tab,Cmt P.symb_subprf_equiv,Tab]) n,
	  INCR_TOP_NUMS];

fun SINCE_EQ s n = () $$ (SINCE_EQ' s n);


fun SINCE_QED' s n top =
    EVERY[NR_CMT (s) n,
	  CMT [Tab,Cmt P.symb_subprf_refine,Tab,Cmt "True"] n,
	  POP_NUMS];

fun SINCE_QED s n = () $$ (SINCE_QED' s n);

val SINCE_END  = POP_NUMS;



(* *********************************************************************** *)
(*									   *)
(* Hiding Standard Operations of the Goal-Package with extended versions   *)
(*									   *)
(* *********************************************************************** *)

fun get_thyname thy =(!(hd(#stamps(Sign.rep_sg(sign_of thy)))));


fun goalw thy rths agoal = 
    (init_num ();
     new_proof();
     punch_proof_item_start ();
     let val t = Goals.goalw thy rths agoal
     in  by(CMTGOAL [Cmt"To show:"] 1); t end);


fun goal thy = goalw thy [];

fun prove_goalw thy rths agoal tacsf =
(* anonymous, (=dangerous! unstable!) name generation *)
    (init_num ();
     new_proof ();
     punch_proof_item_start ();
     let val t       = Goals.prove_goalw thy rths agoal tacsf;
     in  punch_proof_item_end (dot_number()^" ")
				(get_thyname(theory_of_thm (topthm())));
	 punch_thm_obj (dot_number(),"",lemma,short,get_docstate(),t);
	 t
     end);


fun prove_goal thy = prove_goalw thy [];



fun bind_thm (name, thm) =
 (qed_thm := standard thm;
  store_thm (name, standard thm);
  punch_thm_obj (name,name,lemma,short,get_docstate(),!qed_thm);
  use_string ["val " ^ name ^ " = !qed_thm;"]);

fun QED(name,long_name,class,style) =
 (qed_thm := result ();
  store_thm (name, !qed_thm);
  punch_proof_item_end (name^" ")(get_thyname(theory_of_thm (!qed_thm)));
  use_string ["val " ^ name ^ " = !qed_thm;"];
  punch_thm_obj (name,long_name,class,style,get_docstate(),!qed_thm)
);

fun qed name = QED(name,name,lemma,short);

fun qed_goalw name thy rths agoal tacsf =
 (qed_thm := Goals.prove_goalw thy rths agoal tacsf;
  store_thm (name, !qed_thm);
  punch_proof_item_end (name^" ")(get_thyname(theory_of_thm (!qed_thm)));
  use_string ["val " ^ name ^ " = !qed_thm;"];
  punch_thm_obj (name,name,lemma,short,get_docstate(),!qed_thm)
  );

fun qed_goal name thy agoal tacsf = qed_goalw name thy [] agoal tacsf

end;

end;
