(* ******************************************************************** *)
(*  									*)
(* 	Project		: Ida - Isabelle's documentation assistant	*)
(*  	Author 		: B.Wolff					*)
(*  	Affiliation 	: University of Bremen, 			*)
(*			  Bremen Institute of Safe Systems (BISS)	*)
(*   	This file	: Instantiation for .rtf - production		*)
(*			  Format supported by Microsoft Word ...	*)
(*  									*)
(*  	$Date: 1997/09/13 13:40:12 $					*)
(*      $Revision: 1.6 $						*)
(*  									*)
(*  ******************************************************************* *)


(* General remarks for the creation of external syntaxes (alias paraphrasings):

a) The paraphrasing for types is done with an (type-different)
   instantiation list distinct from the one for the operators.

b) Note that Isabelle itself translates syntactical pragmas. The
   creation of an external syntax has to take this into account.
   According to the Isa-Ref-Man (Section "Binders" and "Infixes")
   Binders were translated from:

	c :: (t1 => t2) => t3		(binder "Q" [pb] p)

   into 
	c :: (t1 => t2) => t3	
	"Q":: [idts,t2] => t3		("(3Q_./ _)" [0, pb] p)

   Both declarations have to be overridden in order to define
   external paraphrasings for eta-reduced and standard cases.

   Infixes follow the translation scheme:

	c :: [t1, t2] => t3		(infixl p)

   into:
	"op c" :: [t1, t2] => t3	("(_ c/ _)" [p,p+1] p)

   Now, "Q" and "op c" are the *internal key's* Isabelle uses to
   represent terms. Ida is based on the possibility to alter
   the mapping from them to the template - to associate another
   mixfix to the internal key.
   Thus, overrides of just "c" wont have any effect! It is *necessary*
   to make the above translations by hand in order to get a hand
   on the internal key. (Some aux-funs could facilitate this, though).
   The restoring of the priorityinformation is of importance for the 
   generation of brackets...

c) A general technical problem of paraphrasings is the fact that
   one has to do with a composition on maps on strings. Each string
   language has its own special character and quoting mechanism, 
   usually with an own meta-character:

   There is:
   ML	    : \	: \\ \n \t for \, tab and carriage return.
   Isabelle : ' : '(, '), '', '_, '/
   RTF      : \ : \{  \}

   As a consequence, the string to be passed through the whole
   Isabelle/ida-macinery is \\{\\} for the empty set !!!

*)



structure sgml_instance  : PUNCH_LANG =

struct

  fun repstring a 0 = ""
    | repstring a 1 = a
    | repstring a k =
      if k mod 2 = 0 then repstring(a^a) (k div 2)
                     else repstring(a^a) (k div 2) ^ a;

  val right_margin_init = 80;
  val left_margin_init = 0;
  val indent_depth = 8;

  (* The followin strings are just temporary, minimal settings... *)
  val short_num_bracket_left  = "XXX";
  val short_num_bracket_right = "XXX";
  val symb_subprf_refine = "XXX";
  val symb_subprf_equiv  = "XXX";


  fun blanks(l,c)(il)  = (repstring " " il);
  fun newline(il)(l,c) = "\n" ^ (repstring " " il);;

end;

structure SGML = Doc_punch (sgml_instance);



(* ******************************************************************** *)
(*  									*)
(* 	Building up the paraphrasing library along the HOL-library	*)
(*  									*)
(* ******************************************************************** *)

(* This paraphrasing table was originally generated for Isabelle94-6 and
   evolved from there by hand. It is highly redundant since it contains
   mostly paraphrasings with itself (identity). This was done in order
   to present a template for simplifying
   the creation of other tables with completely different paraphrasings. 
   (eg. LaTeX)
*)


local open SGML Syntax in

val typesT = Type ("types", []);   (* copied from type_ext.ML *)


val it = attach_ext_syntax CPure.thy
[ ("fun",[typeT, typeT] ---> typeT,       Mixfix (
		"(_(2) <FONT SYMB>\\''ae<'\\FONT>/ _)", [1, 0], 0)),
  ("_bracket",[typesT, typeT] ---> typeT, Mixfix (
  		"([_]/(2) <FONT SYMB>\\''ae<'\\FONT> _)", [0, 0], 0))
 (*
   Mfix ("(_/ => _)",   [typeT, typeT] ---> typeT,     "fun", [1, 0], 0),
   Mfix ("([_]/ => _)", [typesT, typeT] ---> typeT,    "_bracket", [0, 0], 0),
 *)
]
[
(* Mixfix.pure_syntax : *)
  ("_lambda",   "[idts, 'a] => logic",            Mixfix (
  		"(3(1)<FONT SYMB>l<'\\FONT>_./ _)", [], 0)),
  ("_abs",      "'a",                             NoSyn),
  ("_args",     "['a, args ] => args",  	  Delimfix "_,/ _"),
  ("_idtyp",    "[id, type] => idt",              Mixfix ("_::_", [], 0)),
  ("_idts",     "[pttrn, idts] => idts",          Mixfix ("_/ _", [1, 0], 0)),
  ("_aprop",    "aprop => prop",                  Delimfix "PROP _"),
  ("_asms",     "[prop, asms] => asms",           Delimfix "_;/ _"),
  ("_bigimpl",  "[asms, prop] => prop",           Mixfix(
  		"((3[| _ |]) ==> _)",[0, 1], 1)),
  ("_ofclass",  "[type, logic] => prop",          Delimfix 
  		"(1OFCLASS/(1'(_,/ _')))"),
  ("_K",        "'a",                             NoSyn),
  
(* proto_pure aus sign.ML: *)
  ("==", "['a::{}, 'a] => prop",  Mixfix (
  		"(_ (1)<FONT SYMB>\\''ba<'\\FONT>/ _)", [3, 2], 2)),
  ("=?=", "['a::{}, 'a] => prop", Mixfix ("(_ =?=/ _)", [3, 2], 2)),
  ("==>", "[prop, prop] => prop", Mixfix (
  		"(_ (2)<FONT BOLD SYMB>\\''de<'\\FONT>/ _)",[2,1],1)),
(*  ("all", "('a => prop) => prop", Binder ("!! ", 0, 0)), *)
  ("!!", "[idts, prop] => prop", Mixfix (
  		"(3  (1)<FONT BOLD SYMB>\042<'\\FONT>_./ _)", [0,0], 0)),
  ("TYPE", "'a itself", NoSyn)
];


val it = attach_ext_syntax HOL.thy
[]
[
 ("Trueprop", "bool  => prop ", Mixfix ("(_)", [], 5)),
 ("not", "bool  => bool ", Mixfix (
 		"(1)<FONT SYMB>\216<'\\FONT>_", [40], 40)),
 ("True", "bool ", NoSyn),
 ("False", "bool ", NoSyn),
 ("If", "[bool,'a,'a] =>'a", Mixfix ("(if (_)/ then (_)/ else (_))", [], 10)),
 ("Inv", "('a   => 'b  ) => ('b   => 'a  )", NoSyn),
 ("Eps", "('a   => bool ) => 'a  ", NoSyn),
(* ("All", "('a   => bool ) => bool ", Binder ("! ", 10, 10)), *)
 ("! ", "[idts, bool] => bool", Mixfix(
 		"(3(1)<FONT SYMB>\034<'\\FONT>_./ _)", [0,10], 10)),
(* ("Ex", "('a   => bool ) => bool ", Binder ("? ", 10, 10)),  *)
 ("? ", "[idts, bool] => bool", Mixfix(
 		"(3(1)<FONT SYMB>\036<'\\FONT>_./ _)", [0,10], 10)),
(* ("Ex1", "('a   => bool ) => bool ", Binder ("?! ", 10, 10)), *)
 ("?! ", "[idts, bool] => bool", Mixfix(
 		"(3(2)<FONT SYMB>\036<'\\FONT>!_./ _)", [0,10], 10)),
 ("Let", "['a, 'a => 'b] => 'b", NoSyn),
 ("=", "['a, 'a] => bool", Infixl 50),
 ("op &", "[bool, bool] => bool", Mixfix (
 		"(_(2) <FONT SYMB>\217<'\\FONT>/ _)", [36, 35], 35)),
 ("op |", "[bool, bool] => bool", Mixfix (
 		"(_(2) <FONT SYMB>\218<'\\FONT>/ _)", [31, 30], 30)),

 ("op o", "['b => 'c, 'a => 'b, 'a] => 'c", Mixfix (
 		"(_(1)<FONT SYMB>\176<'\\FONT>/ _)", [55, 56], 55)),
 ("op -->", "[bool, bool] => bool", Mixfix (
 		"(_(1)<FONT SYMB>\222<'\\FONT>/ _)", [26, 25], 25)),
 ("+", "['a :: plus, 'a] => 'a", Infixl 65),
 ("-", "['a :: minus, 'a] => 'a", Infixl 65),
 ("*", "['a :: times, 'a] => 'a", Infixl 70),

 ("op ~=", "['a,'a] => bool ",Mixfix(
 		"(_(2) <FONT SYMB>\185<'\\FONT> _)", [50,51], 50)),
 ("@Eps", "[pttrn , bool ] => 'a  ", Mixfix (
 		"(3(1)<FONT SYMB>\101<'\\FONT> _./ _)", [], 10)),
 ("*All", "[idts , bool ] => bool ", Mixfix (
 		"(3(1)<FONT SYMB>\034<'\\FONT>_./ _)", [], 10)),
 ("*Ex", "[idts , bool ] => bool ", Mixfix (
 		"(3(1)<FONT SYMB>\036<'\\FONT>_./ _)", [], 10)),
 ("*Ex1", "[idts , bool ] => bool ", Mixfix ("(3(2)<FONT SYMB>\036<'\\FONT>!_./ _)", [], 10)),
 ("_bind", "[pttrn , 'a  ] => letbind ", Mixfix ("(2_ =/ _)", [], 10)),
 ("_binds", "[letbind,letbinds]=>letbinds", Mixfix ("_;/ _", [], max_pri)),
 ("_Let", "[letbinds , 'a  ] => 'a  ", Mixfix ("(let (_)/ in (_))", [], 10)),
 ("@case", "['a  , cases_syn ] => 'b  ", Mixfix ("(case _ of/ _)", [], 10)),
 ("@case1", "['a  , 'b  ] => case_syn ", Mixfix ("(2_ =>/ _)", [], 10)),
 ("@case2", "[case_syn,cases_syn]=>cases_syn",Mixfix("_/ | _", [], max_pri))
];



val it = attach_ext_syntax Ord.thy
[]
[
(* Ord *)
 ("<", "['a :: ord , 'a  ] => bool ", Infixl 50),
 ("op <=", "['a :: ord , 'a  ] => bool ", Mixfix ("(_ (1)<FONT SYMB>\163<'\\FONT> / _)", [50, 51], 50)),
 ("mono", "['a :: ord  => 'b :: ord ] => bool ", NoSyn),
 ("min", "['a :: ord , 'a  ] => 'a  ", NoSyn),
 ("max", "['a :: ord , 'a  ] => 'a  ", NoSyn)
];


val it = attach_ext_syntax Set.thy
[]
[("{}", "'a  set", Mixfix ("(1)<FONT SYMB>\198<'\\FONT>", [], max_pri)),
 ("insert", "['a  , 'a  set] => 'a  set", NoSyn),
 ("Collect", "('a   => bool ) => 'a  set", NoSyn),
 ("Compl", "('a  set) => 'a  set", NoSyn),
 ("op Int", "['a  set, 'a  set] => 'a  set", 
	Mixfix ("(_(2) <FONT SYMB>\199<'\\FONT>/ _)",[70,71],70)),
 ("op Un", "['a  set, 'a  set] => 'a  set",  
	Mixfix ("(_(2) <FONT SYMB>\200<'\\FONT>/ _)",[65,66],65)),
 ("UNION", "['a set, 'a => 'b set] => 'b  set", NoSyn),
 ("INTER", "['a set, 'a => 'b set] => 'b  set", NoSyn),
 ("UNION1", "['a => 'b  set] => 'b  set", Binder ("UN ", 10, 10)),
 ("INTER1", "['a => 'b  set] => 'b  set", Binder ("INT ", 10, 10)),
 ("Union", "(('a set) set) => 'a  set", 
	Mixfix ("(1)<FONT SYMB>\199<'\\FONT>(_)",[],10)),
 ("Inter", "(('a set) set) => 'a  set", 
	Mixfix ("(1)<FONT SYMB>\200<'\\FONT>(_)",[],10)),
 ("Pow", "'a  set => 'a  set set", 
	Mixfix ("(1)<FONT SYMB>\195<'\\FONT>(_)",[0], max_pri)),
 ("range", "('a   => 'b  ) => 'b  set", NoSyn),
 ("Ball", "['a  set, 'a   => bool ] => bool ", NoSyn),
 ("Bex", "['a  set, 'a   => bool ] => bool ", NoSyn),
 ("inj", "('a   => 'b  ) => bool ", NoSyn),
 ("surj", "('a   => 'b  ) => bool ", NoSyn),
 ("inj_onto", "['a   => 'b  , 'a  set] => bool ", NoSyn),
 ("``", "['a   => 'b  , 'a  set] => ('b  set)", Infixl 90),
 ("op :", "['a  , 'a  set] => bool ", 
	Mixfix ("(_(2) <FONT SYMB>\206<'\\FONT>/ _)",[50,51],50)),
 ("UNIV", "'a  set", NoSyn),
 ("op ~:", "['a  , 'a  set] => bool ", 
	Mixfix ("(_(2) <FONT SYMB>\207<'\\FONT>/ _)",[50,51],50)), 
 ("@Finset", "args  => 'a  set", Mixfix ("{(_)}", [], max_pri)), 
 ("@Coll", "[pttrn , bool ] => 'a  set",Mixfix("(1{_./ _})", [], max_pri)),
 ("@SetCompr","['a,idts,bool]=>'a set",Mixfix("(1{_ |/_./ _)",[],max_pri)),
 ("@INTER", "[pttrn,'a set,'b set]=>'b set", 
	Mixfix ("(3(1)<FONT SYMBOL>\199<'\\FONT> _:_./ _)",[],10)), 
 ("@UNION", "[pttrn,'a set,'b set]=>'b set", 
	Mixfix("(3(1)<FONT SYMB>\200<'\\FONT>_:_./ _)",[],10)),
 ("@Ball", "[pttrn,'a set,bool] => bool ", Mixfix ("(3(1)<FONT SYMB>\034<'\\FONT> _:_./ _)", [], 10)),
 ("@Bex","[pttrn,'a set, bool] => bool ", Mixfix ("(3(1)<FONT SYMB>\036<'\\FONT> _:_./ _)", [], 10)),   
 ("*Ball","[pttrn,'a set,bool] => bool ", Mixfix ("(3(1)<FONT SYMB>\034<'\\FONT> _:_./ _)", [], 10)),
 ("*Bex", "[pttrn,'a set,bool] => bool ", Mixfix ("(3(1)<FONT SYMB>\036<'\\FONT> _:_./ _)", [], 10))
];


val it = attach_ext_syntax Prod.thy
[  ("*",[typeT, typeT] ---> typeT, 
    Mixfix ("(_(2) <FONT SYMBOL>\180<'\\FONT>/ _)", [1, 0], 0))
]
[("Pair_Rep", "['a,'b] => ['a, 'b] => bool", NoSyn),
 ("fst", "'a * 'b => 'a", NoSyn),
 ("snd", "'a * 'b => 'b", NoSyn),
 ("split", "[['a, 'b] => 'c, 'a * 'b] => 'c", NoSyn),
 ("prod_fun", "['a => 'b, 'c => 'd, 'a * 'c] => 'b * 'd", NoSyn),
 ("Pair", "['a, 'b] => 'a * 'b", NoSyn),
 ("Sigma", "['a set, 'a => 'b set] => ('a * 'b) set", NoSyn),
 ("()", "unit ", Mixfix ("'(')", [], max_pri)),

 ("@Tuple", "['a, args] => 'a * 'b", Mixfix ("(1'(_,/ _'))", [], max_pri)),
 ("@pttrn", "[pttrn , pttrns ] => pttrn ", Mixfix ("'(_,/_')", [], max_pri)),
 ("@pttrns","[pttrn , pttrns ] => pttrns ",Mixfix ("_,/_", [], max_pri)),
 ("@Sigma","[idt,'a set,'b set]=>('a*'b)set",Mixfix ("(3(1)<FONT SYMBOL>S<`\\FONT>_:_./ _)",[],10)),
 ("@Times","['a set,'a=>'b set]=>('a*'b) set",Mixfix("_ Times _",[81,80], 80))
];



val it = attach_ext_syntax Lfp.thy
[]
[("lfp", "['a  set => 'a  set] => 'a  set", NoSyn)
];



val it = attach_ext_syntax Gfp.thy
[]
[("gfp", "['a  set => 'a  set] => 'a  set", NoSyn)
];



val it = attach_ext_syntax Relation.thy
[]
[("id", "('a * 'a)set", NoSyn),
 ("O", "[('b * 'c)set, ('a * 'b)set] => ('a * 'c)set", Infixr 60),
 ("trans", "('a * 'a)set => bool", NoSyn),
 ("converse", "('a*'b) set => ('b*'a) set", NoSyn),
 ("^^", "[('a*'b) set,'a set] => 'b set", Infixl 90),
 ("Domain", "('a*'b) set => 'a set", NoSyn),
 ("Range", "('a*'b) set => 'b set", NoSyn)
];



val it = attach_ext_syntax Trancl.thy
[]
[("rtrancl", "('a * 'a)set => ('a * 'a)set", Mixfix ("(_^*)", [100], 100)),
 ("trancl", "('a * 'a)set => ('a * 'a)set", Mixfix ("(_^+)", [100], 100)),

 ("reflcl", "('a*'a)set => ('a*'a)set", Mixfix ("(_^=)", [100], 100))
];



val it = attach_ext_syntax WF.thy
[]
[("wf", "('a * 'a)set => bool", NoSyn),
 ("cut", "('a => 'b) => ('a * 'a)set => 'a => 'a => 'b", NoSyn),
 ("is_recfun","('a*'a)set=>(('a=>'b)=>('a=>'b))=>'a=>('a=>'b)=>bool", NoSyn),
 ("the_recfun","('a*'a)set =>(('a=>'b) => ('a=>'b)) => 'a => 'a => 'b", NoSyn),
 ("wfrec", "('a * 'a)set => (('a=>'b) => ('a=>'b)) => 'a => 'b", NoSyn)
];




val it = attach_ext_syntax Nat.thy
[]
[("Zero_Rep", "ind ", NoSyn),
 ("Suc_Rep", "ind  => ind ", NoSyn),
 ("0", "nat ", Mixfix ("0", [], max_pri)),
 ("1", "nat ", Mixfix ("1", [], max_pri)),
 ("2", "nat ", Mixfix ("2", [], max_pri)),
 ("Suc", "nat  => nat ", NoSyn),
 ("nat_case", "['a  , nat  => 'a  , nat ] => 'a  ", NoSyn),
 ("pred_nat", "(nat * nat) set", NoSyn),
 ("nat_rec", "['a  , [nat , 'a  ] => 'a  , nat ] => 'a  ", NoSyn),
 ("Least", "(nat  => bool ) => nat ", Binder ("LEAST ", 10, 10))
];


val it = attach_ext_syntax Arith.thy
[]
[("pred", "nat  => nat ", NoSyn),
 ("div", "[nat , nat ] => nat ", Infixl 70),
 ("mod", "[nat , nat ] => nat ", Infixl 70)
];


val it = attach_ext_syntax RelPow.thy
[]
[("^", "('a * 'a) set => nat => ('a * 'a) set", Infixr 100)
];


val it = attach_ext_syntax Finite.thy
[]
[("Fin", "'a  set => 'a  set set", Mixfix ("Fin (_)",[0], max_pri))
];




val it = attach_ext_syntax Sum.thy
[]
[("Inl_Rep", "['a, 'a, 'b, bool] => bool ", NoSyn),
 ("Inr_Rep", "['b, 'a, 'b, bool ] => bool ", NoSyn),
 ("Inl", "'a => 'a + 'b", NoSyn),
 ("Inr", "'b => 'a + 'b", NoSyn),
 ("sum_case", "['a => 'c, 'b => 'c, 'a + 'b] => 'c", NoSyn),
 ("plus", "['a set, 'b set] => ('a + 'b) set", Infixr 65),
 ("Part", "['a set, 'b => 'a] => 'a set", NoSyn)
];




val it = attach_ext_syntax Univ.thy
[]
[("apfst", "['a=>'c, 'a*'b] => 'c*'b", NoSyn),
 ("Push", "[nat, nat => nat] => (nat => nat)", NoSyn),
 ("Push_Node", "[nat, 'a node] => 'a node", NoSyn),
 ("ndepth", "'a node => nat", NoSyn),
 ("Atom", "('a+nat) => 'a item", NoSyn),
 ("Leaf", "'a => 'a item", NoSyn),
 ("Numb", "nat => 'a item", NoSyn),
 ("$", "['a item, 'a item] => 'a item", Infixr 60),
 ("In0", "'a item => 'a item", NoSyn),
 ("In1", "'a item => 'a item", NoSyn),
 ("ntrunc", "[nat, 'a item] => 'a item", NoSyn),
(*
 ("<*>", "['a item set, 'a item set] => 'a item set", Infixr 80),
 ("<+>", "['a item set, 'a item set] => 'a item set", Infixr 70),
 *)
 ("op <*>", "['a item set, 'a item set] => 'a item set", Mixfix ("(_(2) <FONT SYMBOL>\197<'\\FONT> _)", [81,80], 80)),
 ("op <+>", "['a item set, 'a item set] => 'a item set", Mixfix ("(_(2) <FONT SYMBOL>\196<'\\FONT> _)", [71,70], 70)),

 ("Split", "[['a item, 'a item] => 'b  , 'a item] => 'b  ", NoSyn),
 ("Case", "[['a item] => 'b, ['a item] => 'b, 'a item] => 'b", NoSyn),
 ("diag", "'a set => ('a * 'a)set", NoSyn),
 ("<**>","[('a item*'a item)set , ('a item*'a item)set]=>('a item*'a item)set", Infixr 80),
 ("<++>", "[('a item*'a item)set , ('a item*'a item)set]=>('a item*'a item)set", Infixr 70)
];



val it = attach_ext_syntax Sexp.thy
[]
[("sexp", "'a  item set", NoSyn),
 ("sexp_case", "['a=>'b,nat=>'b,['a item,'a item]=>'b,'a item] =>'b", NoSyn),
 ("sexp_rec","['a item,'a=>'b, nat=>'b,['a item,'a item,'b,'b]=>'b]=>'b", NoSyn),
 ("pred_sexp", "('a item * 'a item)set", NoSyn)
];


val it = attach_ext_syntax List.thy
[]
[("[]", "'a  list", Mixfix ("[]", [], max_pri)), 
 ("#", "['a, 'a list] => 'a list", Infixr 65), 
 ("@", "['a  list, 'a  list] => 'a list", Infixr 65),
 ("drop", "[nat, 'a  list] => 'a list", NoSyn),
 ("filter", "['a => bool, 'a list] => 'a  list", NoSyn),
 ("flat", "'a list list => 'a  list", NoSyn),
 ("foldl", "[['b, 'a] => 'b, 'b, 'a list] => 'b", NoSyn),
 ("hd", "'a list => 'a", NoSyn),
 ("length", "'a list => nat ", NoSyn),
 ("setOfList", "('a list => 'a set)", NoSyn),
 ("list_all", "('a => bool ) => ('a list => bool)", NoSyn),
 ("map", "('a => 'b) => ('a list => 'b list)", NoSyn),
 ("mem", "['a, 'a list] => bool", Infixl 55),
 ("nth", "[nat, 'a list] => 'a", NoSyn),
 ("take", "[nat, 'a list] => 'a list", NoSyn),
 ("tl", "'a list => 'a list", NoSyn),
 ("ttl", "'a list => 'a list", NoSyn),
 ("rev", "'a list => 'a list", NoSyn),

 ("@list", "args  => 'a  list", Mixfix ("[(_)]", [], max_pri)),
 ("@Alls", "[idt,'a list, bool]=>bool",Mixfix ("(2Alls _:_./ _)", [], 10)),
 ("@filter","[idt,'a list,bool]=>'a list",Mixfix ("(1[_:_ ./ _])",[],max_pri))
];

 
end;
 


(* ******************************************************************** *)
(*  									*)
(* 	Checking the content of the data-base				*)
(*  									*)
(* ******************************************************************** *)

print_depth 50;

map fst (Symtab.dest(!SGML.syn_db));

Syntax.infixes_of 2 (SGML.get_external_syntax HOL.thy);
 
print_depth 10;





(* ******************************************************************** *)
(*  									*)
(* 	Miscellaneous:							*)
(*  									*)
(* ******************************************************************** *)





