(* ***************************************************************************

   $Source: /home/uniform/rep/isa_gui/isa_toolkit/focus.sml,v $

   Focus -- the focus package
   
   $Date: 1997/08/22 15:05:07 $
   $Revision: 1.8 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)
	

signature FOCUS = 
sig
    (* This type represents (proof)-states. *)
    type state

    (* selected subgoal *)
    val selSubgoal : unit -> int TkTypes.option

    (* selected subterm aka. the focus
     * The string is the path by which string_of_prfstate (below) marks
     * the respective subterms with the <SEL path> tag 
     *)
    val selFocus   : unit -> string TkTypes.option

    (*  Display a new proofstate. The first argument _must_ be a text widget. 
     *  The second argument is the current proof state.
     *)
    val display : TkTypes.WidId* state-> unit

    (* resets the focus, i.e. nothing is selected. Useful after a new 
     * proof has started etc. *)
    val reset   : unit-> unit

end
   

functor Focus ((* Render the current proof state as a tagged term.
		* Valid tags are 
		*    <SUB integer> to denote subgoals
		*    <SEL path> to denote paths into a term
		*)
	        type prfstate
		val string_of_prfstate : prfstate-> string
	       ) : sig include FOCUS 
                       sharing type state= prfstate
		   end =

struct

    open TkTypes SmlTk Lib.List

    fun debugmsg m = Debug.print 20 ("Focus: "^m)


    (* --- The Focus/Selection -------------------------------------------- *)

    type state= prfstate

    val selGoal    = ref (NONE: (int* AnnId) option)
    val selSubterm = ref (NONE: (string* AnnId) option)

    (* reset focus *)
    fun reset() =  
	(selGoal:= NONE; selSubterm:= NONE)

    (* mark a subgoal: draw a box around it *)
    fun markSubgoal(wid, num, aid) = 
	(addAnnotationConf wid aid [Relief Sunken];
	 selGoal := SOME (num, aid);
	 debugmsg ("Subgoal #"^(Int.toString num)^"selected."))

    (* unmark a subgoal: remove the box *)
    fun unmarkSubgoal wid =
	(case (!selGoal) of
	    SOME (num, aid) => addAnnotationConf wid aid [Relief Flat]
	  | NONE          => ();
	 selGoal := NONE;
	 debugmsg ("Subgoal deselected."))

    (* mark a subterm: invert it *)
    fun markSubterm(wid, path, aid)= 
	(addAnnotationConf wid aid [Foreground Red];
	 selSubterm := SOME (path, aid);
	 debugmsg ("Subterm "^path^" selected.\n"))
	 
    (* unmark a subterm. 
     * Since one cannot delete a configuration, and adding a [Foregrnd. Black]
     * would lead to wrong results (if a superterm of this one is selected,
     * this term would still remain black), we have to get the whole annotation
     * delete it, reset its configuration, and add it back on.
     *)
    fun unmarkSubterm wid =
	(case (!selSubterm) of
	    SOME (path, aid) => 
		let val ann   =  getAnnotation wid aid
		    val nuconf=  filter (fn (Foreground _) => false
				                       | _ => true) 
			                (selAnnotationConf ann)
		in  (delAnnotation wid aid; 
		     addAnnotation wid (updAnnotationConf ann nuconf);
		     debugmsg ("Subterm "^path^" deselected.\n"))
		end
          | NONE          => ();
	 selSubterm := NONE
	)

    fun mkSubgoalTag(wid, num, wh) =
	let val aid = newAnnotationId()
	    val binds= [Bind("<Button-2>", fn _ => 
			     (markSubgoal(wid, num, aid);
			      unmarkSubterm wid))]
	in  TATag(aid, wh, [Borderwidth 2], binds)
	end

    fun mkSelTag(wid, path, wh) =
	let val aid= newAnnotationId()
	    val binds = [Bind("<Button-2>", 
			      fn _ => (unmarkSubterm wid;
				       markSubterm(wid, path, aid)))]
	in  TATag(aid, wh, [], binds)
	end


    exception ParsingError of string

    structure SelectionBindings = 
	struct
	    exception BTagError of string 

	    type widgetinfo= WidId

	    val parsingError = ParsingError

	    datatype bindTag= selTag | sgTag 

	    fun matchingBTag "sel" = SOME selTag
	      | matchingBTag "sub" = SOME sgTag 
              | matchingBTag  _    = NONE

	    fun annotationForBTag selTag [] _ _ = 
		    raise (BTagError "<SEL> tag with no argument")
	      | annotationForBTag selTag (path::_) txwid where = 
		    mkSelTag(txwid, path, [where])
	      | annotationForBTag sgTag [] _ _ =
		    raise (BTagError "<SUB> tag with no argument")
	      | annotationForBTag sgTag (num::_) txwid where = 
		    mkSubgoalTag(txwid, BasicUtil.atoi num, [where])
	end

    (* instantiate the markup-language parser *)
    structure SelTags= SmlTkMarkup(StdTags(SelectionBindings))


    (* update the text widget displaying the subgoals *)
    fun display(twid, st) =
	let val widgtext = string_of_prfstate st
	    val _ = debugmsg("Widget markup text: "^widgtext)
	    val (txt, ann) = SelTags.getAnnotation twid widgtext
		             handle (ParsingError err) =>
			     (output (std_out, "Warning: Parsing error "^err^"\n");
			      ("", []))
	in
	    ((* reset selected focus/subgoal *)
	     reset();
	     (* then replace contents of text widget *)
	     replaceTextWidText twid txt ann;
	     (* bind <Button-3> on text widget to unmark everything *)
	     addBind twid [Bind("<Button-3>", fn _=> (unmarkSubterm twid;
						      unmarkSubgoal twid))]
	    )
	end

    fun selSubgoal () = 
	case (!selGoal) of
	    NONE => NONE
	  | SOME(x, _) => SOME x

    fun selFocus ()   = 
	case (!selSubterm) of
	    NONE => NONE
	  | SOME(x, _) => SOME x


end 
     


