(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/isa_toolkit/history.sml,v $
 
   Support for history managment, for both TAS and IsaWin. 
  
   $Date: 1998/09/18 15:02:55 $
   $Revision: 1.15.4.1 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

signature STATE =
    sig
	type state
	type opn
       
	val apply   : opn* state-> state
	val print   : opn-> string	  
    end


signature HISTORY= 
    sig
	type opn
	type state
	type history

	(* Access to the history. 
	 * Note subtle difference between exec and replace-- replace disnae
	 * discard the future. This may lead to a local fracturisation of
	 * the space-time continuum-- in other words, the subsequent tactics
	 * which will have worked before may now fail.
	 * Morale: don't mess around with history. 
	 *) 
	val forward   : history-> history
	val backward  : history-> history
	val exec      : history* opn-> history
	val replace   : history* opn-> history
	val cur_state : history-> state

	(* Print the history in a meaningful fashion. This gets three arg's,
	 * the history itself, a function to render a state and a function
	 * to render an operation (transforming states). Have a terry's at
	 * ../tas/show.sml for a rather elaborate use of this simple function.
	 *)
        val print     : history* (state-> string)* (opn-> string)-> string 

	(* Show the history window. The argument is the title
	 * of the window 
	 *)
	val show      : string* history-> unit
         
	(* start a new history *)
	val new       : state-> history

	(* the proof level *)
	val level     : history-> int

	(* exception of errors occur, typically forward/backward not possible 
	 * The argument is a descriptive error message ready for display.
	 *)
	exception Error of string

	(* External representation of the history 
	 *
	 * The external representation of a history is based on the
	 * asumption that we can give a cheap representation of opns, but
	 * not for states. When converting back an external representation 
	 * of a history, we additionally need the initial state with which
	 * the history started, since this is not recorded. 
	 *)
	type ext_rep = int* opn list

	val rep : history-> ext_rep
	val mk  : ext_rep* state-> history

    end


functor History (structure conf: GENCONF
		 structure state:STATE) : 
    sig
	include HISTORY  
        sharing type state.opn  = opn
	    and type state.state= state 
    end =

struct

   open TkTypes SmlTk Lib.List 

   structure utilWin= UtilWin(conf.uw)

   fun debug msg = Debug.print 20 ("History: "^msg)


    (* --- The History ---------------------------------------------------- *)

    type state= state.state
    type opn  = state.opn

    type history= (state* opn) list* state* opn list
	          (* corresponding to past, present, future *)
	          (* the second component is the present state. 
		   * the first pair has as components pairs of
		   * state and operations, corresponding to the state _before_
		   * the application was invoked
		   *)		   
    	
    fun new s = ([], s, []) : history

    exception Error of string

    (* displaying the history *)

    fun text (past, _, future)=
	let fun histent (i, t) = (makestring (i:int))^". "^(state.print t)^"\n"
	    val n        = length past
	    val pastext  = map histent (ListPair.zip (1 upto n, 
						      rev (map snd past)))
            val futext   = map histent (ListPair.zip ((n+1) upto 
						       (n+1+(length future)), 
					     future))
	    val spc      = "    "
	    val arrow    = "--> "
	in  spc^(BasicUtil.concatWith spc pastext )^arrow^
	        (BasicUtil.concatWith spc futext)
	end
    

    val histWinId   = ref ""

    fun show(title, hist) = 
	if (occursWin (!histWinId)) then ((* already open, don't do anything *))
	else utilWin.display title 400 (SOME conf.ttFont)
	                     (text hist) (fn win=> histWinId := win)

   (* called after succesful execution of a tactic as a result of any
    * of the history functions below.
    * Set the history to nuhist, and update the display 
    *)
    fun update nuhist =
	 (if occursWin (!histWinId) 
	      then utilWin.redisplay (!histWinId) (text nuhist)
	  else ();
	  nuhist)
	      
    (* Execute an operation *)                                     
    fun doBy (opn, past, present, future) =
	(debug ("About to execute "^(state.print opn));
	 update((present, opn)::past, state.apply(opn, present), future)
	)

    fun backward ([], _, _) =
	        raise Error ("At the start of the history.\nCan't go back.\n")
      | backward ((pstate, opn)::past, present, future)=  
	        update(past, pstate, opn:: future)
 
    fun forward (_, _, []) =
		raise Error("At the end of the history.\nCan't go forward.\n")
      | forward (past, pres, oo::more) =  doBy(oo, past, pres, more)
		  

    fun exec ((past, pres, future), opn) = doBy(opn, past, pres, [])
			
    fun replace ((past, pres, []), opn)         = doBy(opn, past, pres, [])
      | replace ((past, pres, _:: future), opn) = doBy(opn, past, pres, future)

    (* --- Print it and other utilities ---------------------------------- *)

    fun cur_state(_, pres, _) = pres

    fun level  h = length (#1 (h: history))

    fun  print ((past, pres, future), print_state, print_op) =
	foldr (fn ((st, opn), str)=> (print_state st)^(print_op opn)^ str)
	           (print_state pres) (rev past)

    (* --- External representations -------------------------------------- *)

    type ext_rep = int* opn list

    fun rep((past, pres, future):history) = 
	         (length past, (rev (map #2 past))@ future)

    fun mk((i, ops), is) =
	let fun playback(i, ops, h as (past, pres, fut))=
	    if i= 0 then (past, pres, ops)
	    else playback(i-1, tl ops, exec(h, hd ops))
		 handle Empty => h 
	                (* shouldn't happen-- means argument corrupted *)
	in playback(i, ops, new is)
	end 	

end
     



(* --- Ancient history --------------------------------------------------- 

   This is the old version of the history module. It is still needed
   with IsaWin, though this will change soon--- at the moment, I don't
   have the time to change IsaWin.                         --cxl

 *)

signature ANCIENT_HISTORY= 

    sig
	(* tactics with a protocol (the ML function executing the tactic)
	 * and a meaningful description *)
	type ptactic= tactic* string* string

	(* the state comprises the current proof_state and a history associated
	 * with it
	 *)
	type state	    
    
	val  save    : unit -> state
	val  restore : state -> unit

	(* Access to the history. 
	 * The argument is the id of a widget which is merely passed to 
	 * upd_prfstate-- in fact there is no need here this be a WidId
	 * at all. 
	 *)
	val forward   : TkTypes.WidId-> unit
	val backward  : TkTypes.WidId-> unit
	val exec      : TkTypes.WidId-> ptactic-> unit
	val replace   : TkTypes.WidId-> ptactic-> unit

	(* Return the past a list of ptactics. Useful to e.g. print it. *)
	val past      : unit-> ptactic list

	(* show the history window. The argument is the title
	 * of the window 
	 *)
	val show      : string-> unit
        
	type theory
	(* start a new proof *)
	(* raises ERROR if it can't parse the second argument *)
	val new       :  theory* string-> state

	(* the proof level *)
	val level     : unit-> int

        (* the last tactic executed *)
	val last      : unit-> ptactic

    end


functor AncientHistory(
		 structure Thy : 
		       sig type theory
			   type proof
			   val restore_proof: proof-> thm list
			   val save_proof   : unit-> proof
			   val goalw        : theory-> thm list-> string-> thm
			                                                   list
		       end

                 (* This function is called every time the proof state changes.
		  * so that the changes can be reflected in the interface.
		  * The arguments are the id of the widget passed to exec/back/
		  * etc. above, the ptactic the execution of which 
                  * resulted in that proofstate, and the current level. 
		  *)
                val upd_prfstate : TkTypes.WidId* 
		                     (tactic*string* string)* int-> unit
		 (* This is the function used to handle errors occuring
		  * when the tactic (given along with its description)
		  * fails.
		  *)
		val error_handle : tactic* string* string-> exn-> unit
		structure conf: GENCONF) : 
    sig include ANCIENT_HISTORY 
	sharing type theory= Thy.theory 
    end =

struct

   open TkTypes SmlTk Lib.List 

   structure utilWin= UtilWin(conf.uw)

   fun debug msg = Debug.print 20 ("History: "^msg)


   type ptactic = tactic* string* string

   val tac_of  : ptactic-> tactic = #1
   val prot_of : ptactic-> string = #2
   val desc_of : ptactic-> string = #3

    (* --- The History ---------------------------------------------------- *)

    type history= ptactic list* ptactic list
    	
    val blank= ([], []) : history

    val cur_hist= ref blank

    fun last   () = hd (#1 (!cur_hist)) handle Empty => (no_tac, "", "")

    fun level  () = length (#1 (!cur_hist))

    (* displaying the history *)

    fun text (past, future)=
	let fun histent (i, t) = (makestring (i:int))^". "^(desc_of t)^"\n"
	    val n = length past
	    val pastext  = map histent (ListPair.zip (1 upto n, rev past))
            val futext   = map histent (ListPair.zip ((n+1) upto 
						       (n+1+(length future)), 
					     future))
	    val spc      = "    "
	    val arrow    = "--> "
	in  spc^(BasicUtil.concatWith spc pastext )^arrow^
	        (BasicUtil.concatWith spc futext)
	end
    

    val histWinId   = ref ""

    fun show title = 
	if (occursWin (!histWinId)) then ((* already open, don't do anything *))
	else utilWin.display title 400 (SOME conf.ttFont)
	                     (text (!cur_hist)) (fn win=> histWinId := win)

   (* called after succesful execution of a tactic as a result of any
    * of the history functions below.
    * Set the history to nuhist, and update the display 
    *)
    fun update(w, pt, nuhist) =
         (cur_hist := nuhist;
	  if occursWin (!histWinId) 
	      then utilWin.redisplay (!histWinId) (text nuhist)
	  else ();
	  upd_prfstate(w, pt, level())
	 )

    (* Execute a tactic. Catches occuring Isabelle execptions.
     * Call update if no exceptions occur (i.e. tactic was succesful).
     *)                                     
    fun doBy (w, pt, nuhist) =
         (debug ("About to execute "^(prot_of pt));
          by (tac_of pt);
          (* execeptions are caught below, so the following only applies
           * to successful tactics                                        
           *)
	  update(w, pt, nuhist)
         ) handle err=> error_handle pt err

    fun backward w = 
	case (!cur_hist)  of
	    ([], future) =>
	          utilWin.warning "At the start of the history.\n\
				   \Can't go back.\n" (K()) 
	  | (pt::past, future) =>
	          (undo();
		   update(w, pt, (past, pt:: future))
		  )
 
    fun forward w  = 
	case (!cur_hist) of
	    (past, []) =>
		utilWin.warning "At the end of the history.\n\
		                \Can't go forward.\n" (K())
          | (past, pt::more) =>
		   (* A bit debatable-- if a tactic fails when going forward
		    * shouldn't the future be empty? *)
		   doBy(w, pt, (pt:: past, more))
		  

    fun exec w pt =
	let val (past, future) = !cur_hist 
	in  doBy(w, pt, (pt:: past, []))
	end
			
    fun replace w pt =
	case (!cur_hist) of
	    (past, []) =>
		doBy(w, pt, (past, [pt]))	
	  | (past, _:: future) =>
		doBy(w, pt, (past, pt:: future))
		       

    fun past () = (#1 (!cur_hist))

	
   (* --- The state -------------------------------------------------- *)

    type state= history* Thy.proof
    type theory= Thy.theory

    fun save () = 
	(!cur_hist, Thy.save_proof())
	
    fun restore (hist, prf) = 
	(cur_hist:= hist; ignore (Thy.restore_proof(prf)))

    fun new(thy, tor) =
	(Thy.goalw thy [] tor; (blank, Thy.save_proof()))

end
