(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/toolkit/std_markup.sml,v $
 
   The sml_tk Standard Markup Language.

   This module offers a standard markup language for use with sml_tk. It is 
   still generic with respect to the bindings, since these need to be 
   compiled rather than generated.

   This has the disadvantage that the standard tags implemented by the
   StdExMarkup module below (eg. em) can't have bindings, and on the
   other hand the binding tags will find it hard to use the
   font-changing tags provided by said module. On the other hand, it
   is a clear separation of concerns.

   $Date: 1997/07/25 09:17:23 $
   $Revision: 1.4 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)



(* The standard Markup language, extendible *)
(* There is a ready-to-use version below    *)


signature BINDTAGS =
    sig       
	type bindTag
	type widgetinfo
	
	(* matchingBTag returns a bindTag matching the given string *)
	val matchingBTag      : string -> bindTag option


	(* annotationForBTag returns an annotation for a given BTag. 
	 * It may raise exception BTagError (below) if an error occurs
	 *)
	val annotationForBTag : bindTag ->string list-> widgetinfo->
	                                (TkTypes.Mark* TkTypes.Mark)-> 
	                                                    TkTypes.Annotation
							    
	exception BTagError of string

	(* The exception to be raised by the parser if an error occurs *)
	val parsingError      : string-> exn

    end

functor StdTags(bindingTags : BINDTAGS) : 
    sig include TAGS
	sharing type bindingTags.widgetinfo= widgetinfo
    end = 

    struct

	open TkTypes SmlTk 

	val error= bindingTags.parsingError

	exception AnnotationError of string

	datatype tag = 
	    (* fonts *)
	    fontTag | 
	    (* raised/lowered boxes *)
	    raiseTag | boxTag  |
	    (* binding tags *)
	    bindTag of bindingTags.bindTag
	    (* derived tags *)
	    (* superTag | subTag *)
	    
	fun matchingTag "font"  = SOME fontTag
	  | matchingTag "raise" = SOME raiseTag
	  | matchingTag "box"   = SOME boxTag
	  (*		
	  | matchingTag "super" = SOME superTag
	  | matchingTag "sub"   = SOME subTag
	   *)
	  | matchingTag  str    = case bindingTags.matchingBTag str of
	                               SOME bt => SOME (bindTag bt)
				     | NONE    => NONE

        fun fontAnnotation args where = 

	    let exception NoConfFont
		
		fun fontConf "bf"    = Bold
		  | fontConf "bold"  = Bold
		  | fontConf "it"    = Italic
		  | fontConf "em"    = Italic
		  | fontConf "tiny"  = Tiny
		  | fontConf "small" = Small
		  | fontConf "large" = Large
		  | fontConf "huge"  = Huge 
		  | fontConf str     = 
		     if BasicUtil.isPrefix "size=" str
		     then let val fstr  = substring(str, 5, (size str)-5)
			      val factor= case (Real.fromString fstr) of
			                       SOME f=> f
					     | NONE  => raise (AnnotationError "No argument for SCALE config")
			  in  Scale factor
			  end
		     else
			 raise NoConfFont	    
		    
		fun fontName "tt"    = Typewriter
		  | fontName "sf"    = SansSerif
		  | fontName "symb"  = Symbol
		  | fontName  _      = raise NoConfFont
		    
		fun foldConfig(c, r) =
		    (fontConf c)::r 
		    handle NoConfFont => r

		fun getFont a =
		    fontName (List.last a)
		    handle NoConfFont => Normalfont
		    handle Empty      => Normalfont
	       
		val font = (getFont args) 
		           (foldr foldConfig ([]: FontConfig list) args)
	    in
		TATag(newAnnotationId(), [where], 
		      [Font font], [])
	    end


	fun getFirstArg nm []     = raise (AnnotationError
					  ("No argument for "^nm^" tag"))
	  | getFirstArg _  (r::_) = r

	type widgetinfo= bindingTags.widgetinfo

	fun annotationForTag fontTag args wi wh = fontAnnotation args wh
	  | annotationForTag raiseTag r wi wh = 
	                     TATag(newAnnotationId(), [wh],
				   (* [Offset (Int.fromString (getFirstArg "RAISE"))] *)
				   [], [])
	  | annotationForTag boxTag r wi wh = 
                             TATag(newAnnotationId(), [wh],
				  (* [Offset (Int.fromString (getFirstArg "BOX"), *)
				  [Relief Groove, Borderwidth 1], [])
	  | annotationForTag (bindTag btag) args wi wh  = 
	                     bindingTags.annotationForBTag btag args wi wh
	    
	    
	fun escape _  = NONE (* as yet -- may want &alpha; etc. *)
	    
	fun warning w = output(std_out, "SmlTk Markup warning: "^w^"\n")
	                (* should use the warning window from utilwin
			 * -- no it bloody shouldn't, since parsing can 
			 *    occur at compile time.
			 *)

    end



structure StdMarkup =
    SmlTkMarkup(StdTags(struct		 
		     type bindTag = unit
		     type widgetinfo= TkTypes.WidId

		     exception BTagError of string
		     val parsingError= Fail

		     fun matchingBTag _ =  NONE
		     fun annotationForBTag () _ _ _ =
			 raise (BTagError 
				"Illegal annotation in annotationForBTag")
			 
		 end))







